//---------------------------------------------------------------------------
#ifndef SQUID_AUTH_LDAP
#define SQUID_AUTH_LDAP
//---------------------------------------------------------------------------
#define PROG "squid_auth_ldap"
#define VERS "2.0.3"
//---------------------------------------------------------------------------
#include <stdio.h>
#include <string.h>
#include <lber.h>
#include <ldap_cdefs.h>
#include <ldap.h>
#include <sys/time.h>

//---------------------------------------------------------------------------
#define OK "OK\n"
#define ERR "ERR\n"

// This defines which attribute should be used for searching for 
// the users.
// Everything should be possible, cn, uid, mail, etc.
// It is possible to override these on either the commandline, or 
// in the configuration file.
#define USRSRCATTR "uid"

#define MEMBERATTR "groupMembership"
// #define BLDUSRSRC  "(&(objectClass=inetOrgPerson) (| (%s=%s)))"
#define BLDUSRSRC  "(&(|(objectClass=inetOrgPerson)(objectClass=alias)) (| (%s=%s)))"
//---------------------------------------------------------------------------
#define OK_NOCFG 1
#define OK_CFG 2
//---------------------------------------------------------------------------

// readconf.h
struct config {
  char ldapServer[200];
  int is_ldapServer;
  int  ldapPort;
  int is_ldapPort;
  char searchBase[512];
  int is_searchBase;
  char pxyGroup[512];
  int is_pxyGroup;
  char confFile[256];
  int is_confFile;
  int logLevel;
  char logFile[256];  
  int is_logFile;
  int logDebug;
  char bindDN[1024];
  int is_bindDN;
  char bindPWD[1024];
  int is_bindPWD;
  char groupAttr[1024];
  char userAttr[1024];
};

int readconf( char* conffile, struct config* );


// ldap_utils.c
LDAP* ldapOpenBind( struct config *conf);
LDAP* ldapOpen( struct config *conf);

int canBind( struct config *conf, LDAP* ld, char *dn, char *passwd );

int findUserDN( LDAP* ld, char* searchBase, char *uid, char *udn );
int getUserDN( LDAP* ld, struct config *conf, char *uid, char *udn );

int getGroupDN( LDAP* ld, struct config *conf, char* grpDN );
int isUserInLDAPgrp( LDAP *ld, char *gdn, char *dn );

int checkUserGroup( LDAP *ld, struct config *conf, char *gdn, char *dn);

// logging.c
#define DEBUG 3
#define WARN  2
#define INFO  1
#define RUN   0


#define LDAPSERVER "LDAPSERVER"
#define LDAPPORT   "LDAPPORT"

#define SEARCHBASE "SEARCHBASE"
#define PROXYGROUP "PROXYGROUP"

#define LOGLEVEL   "LOGLEVEL"
#define LOGFILE    "LOGFILE"

#define BINDDN     "BINDDN"
#define BINDPWD    "BINDPWD"

#define GROUPATTR  "GROUPATTR"
#define USERATTR   "USERATTR"

// options.c
int options( int argc, char* argv[], struct config* );

// logging.c
void StartLog( struct config* );
void _StartLog( );
void StopLog( );
void logging( int ll, const char* fmt, ... );
void startMsg( );
void logMsg( int ll );
void buildMsg( const char* fmt, ...);

// tools.c
void strip( char* );
void head( char* );
void tail( char* );
void clrConfig( struct config *conf);

//---------------------------------------------------------------------------
#endif
//---------------------------------------------------------------------------
