/*
  logging.c
  general function for logging
  
  Author: Casper Pedersen
          cp@c-note.dk

*/

// ------------------------------------------------------------------------
#include <stdio.h>
#include <stdarg.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <syslog.h>


// ------------------------------------------------------------------------
#include "defines.h"

// ------------------------------------------------------------------------
static char _msgBuffer[4096];
static int  _logLevel;

// ------------------------------------------------------------------------
// ------------------------------------------------------------------------
void _StartLog( )
{
  openlog(PROG, LOG_CONS | LOG_PID, LOG_USER );
}

void StartLog( struct config *conf )
{
  StopLog( );
	_logLevel = conf->logLevel;
  if( conf->logDebug == 0 )
    openlog(PROG, LOG_CONS | LOG_PID, LOG_USER );
  else
  if( conf->logDebug == 1 )
  {
    openlog(PROG, LOG_CONS | LOG_PERROR | LOG_PID, LOG_USER );
  }  
}
  
// ------------------------------------------------------------------------
// ------------------------------------------------------------------------
void StopLog( )
{
  closelog( );
}

// ------------------------------------------------------------------------
// ------------------------------------------------------------------------
void logging( int ll, const char* fmt, ... )
{
  char buffer[1024];
  va_list ap;
  va_start( ap, fmt );
  
  vsnprintf( buffer, sizeof( buffer ), fmt, ap );  

  if( ll == DEBUG && _logLevel >= DEBUG )
	{
	  syslog( LOG_INFO, buffer );
	}
	else 
	if( ll == WARN && _logLevel >= WARN )
	{
	  syslog( LOG_INFO, buffer );
	}
	else
	if( ll == INFO && _logLevel >= INFO )
	{
	  syslog( LOG_INFO, buffer );
	}
	else
	if( ll == RUN && _logLevel >= RUN )
	{
	  syslog( LOG_INFO, buffer );		
	}
}

// ------------------------------------------------------------------------
// ------------------------------------------------------------------------
void startMsg( )
{
  _msgBuffer[0] = '\0';
}

// ------------------------------------------------------------------------
// ------------------------------------------------------------------------
void logMsg( int ll )
{
	logging( ll, _msgBuffer );
}

// ------------------------------------------------------------------------
// ------------------------------------------------------------------------
void buildMsg( const char *fmt, ... )
{
  char buffer[1024];
  va_list ap;
  va_start( ap, fmt );
  
  vsnprintf( buffer, sizeof( buffer ), fmt, ap );  
	if( strlen( _msgBuffer ) >= sizeof( _msgBuffer ) )
	{
	  return;
	}
	else
  if( ( strlen( buffer ) + strlen( _msgBuffer ) )  < sizeof( _msgBuffer ) )
	{
	  strcat( _msgBuffer, buffer );  		
	}
	else
	{
	  strncat( _msgBuffer, buffer, sizeof( _msgBuffer ) - strlen( _msgBuffer ) - 1 );
	}
}
// ------------------------------------------------------------------------
// ------------------------------------------------------------------------
