/*
   main.c
   
   Author: Casper Pedersen
           cp@c-note.dk
*/	   

// ------------------------------------------------------------------------
#include <stdio.h>
#include <string.h>
#include <lber.h>
#include <ldap_cdefs.h>
#include <ldap.h>
#include <stdio.h>
#include <sys/time.h>
#include <stdarg.h>
#include <unistd.h>

// ------------------------------------------------------------------------
#include "defines.h"
// ------------------------------------------------------------------------


// ------------------------------------------------------------------------
int getUIDPass (char* buf, char *user, char *pass);

// ------------------------------------------------------------------------
// main
// Main function
// ------------------------------------------------------------------------
int main (int argc, char **argv)
{
	char dnGrp[1024];  // eDirectory "only" support 255 in a CN
	char buf[256];
	char udn[256];

	LDAP *ld = NULL;

	int rc, retVal = 1;
	int chkGroup = 0;
	int opt = 0;
	struct config conf;

	setbuf(stdout,NULL);

	_StartLog( );
	logging( RUN, "%s ver. %s, (c) Casper Pedersen 2001-2002\n", PROG, VERS );
	logging( RUN, "- starting...");

	if( (opt = options( argc, argv, &conf )) == 0 )
	{
   		exit(1);
	}

	if( opt == OK_CFG )
	{
    	if( readconf( conf.confFile, &conf ) == 0 )
      		exit(1);
	}

	StartLog( &conf );

	// Get our self a LDAP connection
	if( (ld = ldapOpenBind( &conf )) == NULL )
	{
		exit (1);
	}

  	// Get FQN of group - crappy code
	if( conf.is_pxyGroup )
	{
    	int ret;
    	logging( RUN, "- find DN for group %s\n", conf.pxyGroup );
    	ret = getGroupDN(ld, &conf, dnGrp );
    	if( ret == -1 )
    		exit(1);
	    else
    	if( ret != 1 )
    	{
      		logging( RUN, "- unable to find group: %s", conf.pxyGroup );
      		exit(1);
    	}
    
    	logging( DEBUG, "- group DN: %s", dnGrp );
    	chkGroup = 1;
	}  
	
	logging( RUN, "- ready" );

	// Main loop
  	while( fgets(buf, 256, stdin) != NULL )
  	{    
    	char user[256];
    	char passwd[256];
    	int getid;
		
    	user[0] = '\0';
    	passwd[0] = '\0';
				
    	startMsg( );	
	
    	getid = getUIDPass(buf, user, passwd);
    	buildMsg( "- usr %s ", user );

    	// If we didn't get both user and passwd - try again.
    	if( getid == 0 )
    	{
      		logging( DEBUG, "- did not get a valide string from the browser, ie. user pass");
      		continue;
    	}
			
    	if( getUserDN( ld, &conf, user, udn ) )
    	{
      		int isInGrp = 0;
      
      		buildMsg( "-> %s ", udn );
    
      		if( chkGroup )
      		{
				if( checkUserGroup( ld, &conf, dnGrp, udn ) )
          			isInGrp = 1;
      		}  
      		else
				isInGrp = 1;        
    
      		if( isInGrp )
      		{
        		retVal = canBind (&conf, ld, udn, passwd);
        		if( retVal == LDAP_SUCCESS ) 
        		{
          			buildMsg( "-> Authenticated!" );
	  				printf(OK);
				}
				else
				{
	  				buildMsg( "-> Not Authenticated!");
          			printf(ERR);	
				}
      		}
      		else
      		{
        		buildMsg( "-> Not in Group %s", udn );			
        		printf(ERR);
      		}
    	}
    	else
    	{
      		buildMsg( "-> Cannot find user!");
      		printf(ERR);
    	}      
    
    	logMsg( INFO );
  	}

	// Should actually never get to this point.
	ldap_unbind (ld);    

	logging( RUN, "%s - %s - stopping", PROG, VERS );
	StopLog();
	return 0;
}


// ------------------------------------------------------------------------
// getUIDPass
// Splits incomming string from Squid into uid and passwd 
// ------------------------------------------------------------------------
int getUIDPass(char *buf, char *user, char *pass)
{
	char *p, *uid, *pwd;

	// Clear buffer for control characters
	// Yes I know that this will not allow control characters as password.	
	tail( buf );
	
#ifdef _DEBUG_SEE_PASSWORD_
	logging( DEBUG, "- user string: |%s|", buf);
#endif

	if ((uid = strtok(buf, " ")) == NULL)
  	{
    	printf(ERR);
    	logging( WARN, "- ERR: user");
    	return 0;
  	}
    
  	if ((pwd = strtok(NULL, "")) == NULL)
  	{
    	printf (ERR);
    	logging( WARN, "- ERR: no password!");
    	return 0;
  	}

	strcpy (user, uid);
	strcpy (pass, pwd);
	
	logging( DEBUG, "- got User: %s", user );	
	// We don't really want to show the password in the log. 
	logging( DEBUG, "- got Password: %s", crypt (pass, "the_answer_is_42") );

  	return 1;
}


