/*
   options.c
   get command options
   
   Author: Casper Pedersen
           cp@c-note.dk
*/	   
// ------------------------------------------------------------------------
#include <stdio.h>
#include <unistd.h>

// ------------------------------------------------------------------------
#include "defines.h"

// ------------------------------------------------------------------------
// ------------------------------------------------------------------------
static void usage( void )
{
	fprintf(stderr,"\n");
	fprintf(stderr,"%s ver. %s, (c) Casper Pedersen 2001-2002\n", PROG, VERS );
	fprintf(stderr,"\n");
	fprintf(stderr,"Usage: squid_auth_ldap -[f config-file] -[Sps[GUgld]] -h\n");
	fprintf(stderr,"       -f cofiguration file\n");
	fprintf(stderr,"       -S ldapServer\n");
	fprintf(stderr,"       -p ldapPort (default 389)\n");
	fprintf(stderr,"       -s searchBase\n");
	fprintf(stderr,"       -g proxyGroup\n");
	fprintf(stderr,"       -G group attribute (default \"groupMembership\")\n");
	fprintf(stderr,"       -U user attribute (default \"uid\")\n");
	fprintf(stderr,"       -l logfile (not implemented - using syslog)\n");
	fprintf(stderr,"       -d loglevel <0..3> (default 0)\n");
	fprintf(stderr,"       -e debug - logs to stderr\n");
	fprintf(stderr,"       -D binddn\n");
	fprintf(stderr,"       -w bindpasswd\n");
	fprintf(stderr,"       -h help\n");
	fprintf(stderr,"\n");
	fprintf(stderr,"- debug levels:\n");
	fprintf(stderr,"        0 - none\n");
	fprintf(stderr,"        1 - info\n");
	fprintf(stderr,"        2 - warnings\n");
	fprintf(stderr,"        3 - debug\n");
	fprintf(stderr,"\n");
	fprintf(stderr,"squid_auth_ldap -f /etc/squid_auth_ldap.conf\n");
	fprintf(stderr,"squid_auth_ldap -S acme_ldap -p 389 -s o=acme -g pxyUsers\n");
	fprintf(stderr,"\n");
}

// ------------------------------------------------------------------------
// ------------------------------------------------------------------------
int options (int argc, char* argv[], struct config* conf)
{
	int opt;
	char msg[2024];
	msg[0] = '\0';
      
	clrConfig( conf );	
	
	opt = getopt( argc, argv, "DSGUewpsgfldh" );
	while( opt != -1 )
  	{
    	switch( opt )
    	{
			case 'G': strcpy( conf->groupAttr, argv[optind] );
							logging(DEBUG,"- groupAttr: %s", conf->groupAttr );
							break;
								
			case 'U': strcpy( conf->userAttr, argv[optind] );
							logging(DEBUG,"- userAttr: %s", conf->userAttr );
							break;
								
			case 'e': 	conf->logDebug = 1;
		 					conf->logLevel = 3;
							logging(DEBUG,"- debug");
							break;
    
			case 'S': 	strcpy( conf->ldapServer, argv[optind] );
							conf->is_ldapServer = 1;
							logging(DEBUG,"- ldapServer: %s ", conf->ldapServer );
							break;
      
			case 'p': 	conf->ldapPort = atoi( (char*)argv[optind]);
							conf->is_ldapPort = 1;
							logging(DEBUG,"- ldapPort: %d ", conf->ldapPort );
							break;

			case 's':	strcpy( conf->searchBase, argv[optind] );
							conf->is_searchBase = 1;
							logging(DEBUG,"- searchBase: %s ", conf->searchBase );
							break;
		
			case 'g':	strcpy( conf->pxyGroup, argv[optind] );
							conf->is_pxyGroup = 1;
							logging(DEBUG,"- pxyGroup: %s ", conf->pxyGroup );
							break;

			case 'f':	strcpy( conf->confFile, argv[optind] );
							conf->is_confFile = 1;
							logging(DEBUG,"- confFile: %s ", conf->confFile );
							break;
								
								
							// I need to write my own logging code at some
							// point, but until that this will not get enabled
			case 'l':	// strcpy( conf->logFile, argv[optind] );
							// conf->is_logFile = 1;
							// logging("OPT logFile: %s\n", conf->logFile );
							break;
    
    		case 'd': 	conf->logLevel = atoi(argv[optind]);
							logging(DEBUG,"- logLevel: %d ", conf->logLevel );
							break;		
		
			case 'D':	strcpy( conf->bindDN, argv[optind] );
							logging(DEBUG,"- bindDN: %s ", conf->bindDN );
    						conf->is_bindDN = 1;
							break;

			case 'w':	strcpy( conf->bindPWD, argv[optind] );
							logging(DEBUG,"- bindPWD: %s ", conf->bindPWD );
    						conf->is_bindPWD = 1;
							break;
		
			case 'h': 
			default : 	usage( ); 
							exit(0);
    	}
    	opt = getopt( argc, argv, "DSGUewpsgfldh" );
	}
  
	if( conf->is_ldapServer && conf->is_searchBase )
	{
		logging(RUN,"- enough params to run!");
	    return( OK_NOCFG );
	}
	else
	if( conf->is_confFile )
  	{
    	logging(RUN,"- enough params to run!");    
    	return( OK_CFG );
	}
  
	usage( );
  
  	return 0;
}
// ------------------------------------------------------------------------
// ------------------------------------------------------------------------
