/*
  readconf.c
  read the configuration file
  
  Author: Casper Pedersen
          cp@c-note.dk
*/
// ------------------------------------------------------------------------
#include "defines.h"

// ------------------------------------------------------------------------
// ------------------------------------------------------------------------
int readconf( char* conffile, struct config* conf )
{
	FILE *fp;
	char buf[512];
	char *p;
	int  line = 0;
	int  found = 0;

	clrConfig( conf );
	  
	fp = fopen( conffile, "r" );
	if( fp == NULL )
	{
		logging( RUN, "- cannot open configuration file: %s", conffile );
		return 0;
	}
  
	while( (fgets(buf, 512, fp)) != NULL)
	{
		char *parm;
    	char *val;
    
    	line++;
    
    	strip(buf);
    
    	if( buf[0] == '#' )
      		continue;
      
    	logging(DEBUG,"- conf. string %s", buf);
    
    	parm = strtok( buf, " " );
    	val = strtok( NULL, "" );
    
    	if( (parm == NULL) || (val == NULL) )
    	{
      		logging(RUN,"- invalid entry in line: %d", line);

			if( parm != NULL )
			{
				logging( RUN, "- %s does not have a value!!", parm );
				logging( RUN, "- exiting.....");
			}

	      	fclose(fp);
      		return 0;
    	}

    	strip( parm );
    	strip( val );

    	logging(DEBUG,"- parm: %s - val: %s", parm, val );
  
    	if( (strcasecmp(LDAPSERVER, parm)) == 0 )
    	{
      		strcpy( conf->ldapServer, val );
      		conf->is_ldapServer = 1;
    	}
    	else
    	if( (strcasecmp(LDAPPORT, parm)) == 0 )
    	{
      		conf->ldapPort = atoi( (char*) val );
     		conf->is_ldapPort = 1;
    	}
    	else
    	if( (strcasecmp(SEARCHBASE, parm)) == 0 )
    	{
      		strcpy( conf->searchBase, val );
      		conf->is_searchBase = 1;
    	}
    	else
   		if( (strcasecmp(PROXYGROUP, parm)) == 0 )
    	{
      		strcpy( conf->pxyGroup, val );
      		conf->is_pxyGroup = 1;
    	} 
    	if( (strcasecmp(BINDDN, parm)) == 0 )
    	{
      		strcpy( conf->bindDN, val );
      		conf->is_bindDN = 1;
    	} 
    	if( (strcasecmp(BINDPWD, parm)) == 0 )
    	{
      		strcpy( conf->bindPWD, val );
      		conf->is_bindPWD = 1;
    	}    	
    	if( (strcasecmp(PROXYGROUP, parm)) == 0 )
    	{
      		strcpy( conf->pxyGroup, val );
      		conf->is_pxyGroup = 1;
    	} 
    	else
   		if( (strcasecmp(LOGFILE, parm)) == 0 )
    	{
      		strcpy( conf->logFile, val );
    	}
    	else
    	if( (strcasecmp(LOGLEVEL, parm)) == 0 )
    	{
      		conf->logLevel = atoi( val );
    	}
		else
		if( (strcasecmp(GROUPATTR, parm)) == 0 )
		{
			strcpy( conf->groupAttr, val );
		}
		else
		if( (strcasecmp(USERATTR, parm)) == 0 )
		{
			strcpy( conf->userAttr, val );
		}    
	} 
  
	fclose(fp);
 
	if( !conf->is_ldapServer )
	{
   		fprintf(stderr,"- missing LDAPSERVER <param> in %s\n", conffile );
    	return 0;
  	}
  	if( !conf->is_ldapPort )
  	{
   		fprintf(stderr,"- missing LDAPPORT <param> in %s\n", conffile );
    	return 0;
  	}
  	if( !conf->is_searchBase )
  	{
    	fprintf(stderr,"- missing SEARCHBASE <param> in %s\n", conffile );
    	return 0;
  	}
  
	return 1;
}

// ------------------------------------------------------------------------
// ------------------------------------------------------------------------
